///
/// A general-purpose Piccolo-based multi-column Tree control.
/// It accepts a TreeModel which defines the number, name and size of columns,
/// as well as the content of the tree.
/// 
/// See model defined by TreeModel and TreeNode interfaces.
/// 
/// Ben Bederson
/// Human-Computer Interaction Lab
/// University of Maryland
/// May, 2005
/// 
using System;
using System.Drawing;
using System.Windows.Forms;

using UMD.HCIL.Piccolo;
using UMD.HCIL.Piccolo.Util;
using UMD.HCIL.Piccolo.Activities;
using UMD.HCIL.PiccoloX.Components;

namespace TreeControl {
	public class TreeControl : UserControl {
		private static int ANIMATION_TIME = 500;

		private TreeModel treeModel;
		private PScrollableControl scrollableControl;
		private PCanvas canvas;
		private Font font;
		private int[] colWidths;
		private int colHeight;

		public TreeControl() {
			font = new Font("Arial", 12);

			scrollableControl = new PScrollableControl();
			canvas = new PCanvas();
			canvas.AnimatingRenderQuality = RenderQuality.HighQuality;
			canvas.InteractingRenderQuality = RenderQuality.HighQuality;
			scrollableControl.Canvas = canvas;
			canvas.Camera.MouseDown += new PInputEventHandler(Camera_MouseDown);
			canvas.PanEventHandler = null;

			Controls.Add(scrollableControl);
		}

		public TreeControl(TreeModel treeModel) : this() {
			TreeModel = treeModel;
		}

		public TreeModel TreeModel {
			get { return treeModel; }
			set {
				treeModel = value;
				ResetView();
			}
		}

		public override Font Font {
			get { return font; }
			set {
				font = value;
				ResetView();
			}
		}

		public int ColHeight {
			get { return colHeight; }
		}

		public int[] ColWidths {
			get { return colWidths; }
		}

		/********************************************************************
		 * INTERNAL IMPLEMENTATION
		 *******************************************************************/

		// When the control gets resized, we want to know so we can change the size
		// of the PScrollableControl and the Canvas
		protected override void SetBoundsCore(int x, int y, int width, int height, BoundsSpecified specified) {
			base.SetBoundsCore(x, y, width, height, specified);

			scrollableControl.Bounds = new Rectangle(x, y, width, height);
			scrollableControl.Canvas.Bounds = new Rectangle(x, y, width, height);
		}

		// Resets the entire view - loading the scene graph with default nodes based on the model
		private void ResetView() {
			canvas.Layer.RemoveAllChildren();

			// First calculate size of columns
			TreeNode header = treeModel.GetColumnHeader();
			int numCols = header.GetColumnCount();
			colWidths = new int[numCols];
			Graphics g = CreateGraphics();
			for (int i=0; i<numCols; i++) {
				string col = header.GetColumn(i);
				StringFormat format = new StringFormat();
				format.FormatFlags = StringFormatFlags.MeasureTrailingSpaces;
				SizeF colSize = g.MeasureString(col, font, new Point(0, 0), format);
				colWidths[i] = (int)Math.Ceiling(colSize.Width);
				if (i == 0) {
					colHeight = (int)Math.Ceiling(colSize.Height);
				}
			}

			// Then create column header
			TreeViewNode headerViewNode = new TreeViewNode(this, treeModel.GetColumnHeader());
			headerViewNode.Header = true;
			canvas.Layer.AddChild(headerViewNode);

			// And create root
			TreeNode root = treeModel.GetRoot();
			TreeViewNode rootViewNode = new TreeViewNode(this, root);
			rootViewNode.TranslateBy(0, ColHeight);
			canvas.Layer.AddChild(rootViewNode);
		}

		// When a contraction is finished, this gets notified - which removes the children
		TreeViewNode targetViewNode = null;
		private void ContractActivityFinished(PActivity activity) {
			if (targetViewNode != null) {
				targetViewNode.RemoveAllChildren();
				targetViewNode = null;
			}
		}

		// Respond to main mouse press in control
		private void Camera_MouseDown(object sender, UMD.HCIL.Piccolo.Event.PInputEventArgs e) {
			if (e.Button == MouseButtons.Right) {
				return;
			}

			PNode node = e.PickedNode;
			if (node is TreeViewNode) {
				TreeViewNode tvn = (TreeViewNode)node;

				if (!treeModel.IsLeaf(tvn.TreeNode)) {
					if (tvn.Open) {
						// If we're closing a node, then contract children
						targetViewNode = tvn;
						foreach (PNode child in tvn) {
							PTransformActivity activity = child.AnimateToMatrix(new PMatrix(), ANIMATION_TIME);
							activity.ActivityFinished = new ActivityFinishedDelegate(ContractActivityFinished);
						}
					
						// Move back nodes below into newly vacant space
						tvn.Open = false;
						int dy = -(int)(tvn.FullBounds.Height - tvn.Bounds.Height);
						PNode target = tvn;
						PNode parent = target.Parent;
						while (parent != canvas.Layer) {
							bool found = false;
							foreach (PNode child in parent) {
								if (found) {
									PMatrix matrix = child.Matrix;
									matrix.TranslateBy(0, dy);
									child.AnimateToMatrix(matrix, ANIMATION_TIME);
								}
								if (child == target) {
									found = true;
								}
							}
							target = parent;
							parent = parent.Parent;
						}
					} else {
						// If we're opening a node, then first we make enough space for the new nodes
						int numChildren = treeModel.GetChildCount(tvn.TreeNode);
						PNode target = tvn;
						PNode parent = target.Parent;
						int dy = ColHeight * numChildren;
						while (parent != canvas.Layer) {
							bool found = false;
							foreach (PNode child in parent) {
								if (found) {
									PMatrix matrix = child.Matrix;
									matrix.TranslateBy(0, dy);
									child.AnimateToMatrix(matrix, ANIMATION_TIME);
								}
								if (child == target) {
									found = true;
								}
							}
							target = parent;
							parent = parent.Parent;
						}

						// Then, we create the new children
						tvn.Open = true;
						dy = ColHeight;
						for (int i=0; i<numChildren; i++) {
							TreeNode child = treeModel.GetChild(tvn.TreeNode, i);
							TreeViewNode childViewNode = new TreeViewNode(this, child);
							childViewNode.Level = tvn.Level + 1;
							tvn.AddChild(childViewNode);
							childViewNode.AnimateToPositionScaleRotation(0, dy, 1, 0, ANIMATION_TIME);
							dy += ColHeight;
						}
					}
				}
			}
		}
	}
}