///
/// The custom Piccolo node used to display a single multi-column node in the tree.
/// Used by TreeControl.
/// 
/// Ben Bederson
/// Human-Computer Interaction Lab
/// University of Maryland
/// May, 2005
/// 
using System;
using System.IO;
using System.Reflection;
using System.Drawing;
using System.Windows.Forms;
using UMD.HCIL.Piccolo;

namespace TreeControl {
	public class TreeViewNode : PNode {
		private static int CTLIMAGE_OFFSET = 20;
		private static int ICON_OFFSET = 20;
		private static int LEVEL_OFFSET = 20;
		private static Image plusImage = null;
		private static Image minusImage = null;
		private static Image folderImage = null;
		private static Image leafImage = null;

		private TreeControl treeControl;
		private TreeNode treeNode;
		private string[] cols;
		private Image ctlImage;
		private Image icon;
		private bool header;
		private bool open = false;
		private int level = 0;
		private Font font;

		public TreeViewNode(TreeControl treeControl, TreeNode treeNode) {
			this.treeControl = treeControl;
			this.treeNode = treeNode;

			if (plusImage == null) {
				LoadImages();
			}
			if (treeControl.TreeModel.IsLeaf(treeNode)) {
				ctlImage = null;
				icon = leafImage;
			} else {
				ctlImage = plusImage;
				icon = folderImage;
			}

			int numCols = treeNode.GetColumnCount();
			cols = new string[numCols];
			for (int i=0; i<numCols; i++) {
				cols[i] = treeNode.GetColumn(i);
			}

			ComputeBounds();
		}

		public Font Font {
			get { return font; }
			set {
				font = value; 
				ComputeBounds();
			}
		}

		private void LoadImages() {
			Assembly assem = treeControl.GetType().Assembly;
			plusImage = new Bitmap(assem.GetManifestResourceStream("TreeControl.images.plus.png"));
			minusImage = new Bitmap(assem.GetManifestResourceStream("TreeControl.images.minus.png"));
			folderImage = new Bitmap(assem.GetManifestResourceStream("TreeControl.images.folder.png"));
			leafImage = new Bitmap(assem.GetManifestResourceStream("TreeControl.images.leaf.png"));
		}

		private void ComputeBounds() {
			int width = CTLIMAGE_OFFSET + ICON_OFFSET;
			for (int colNum=0; colNum<cols.Length; colNum++) {
				string col = cols[colNum];
				width += treeControl.ColWidths[colNum];
			}

			Bounds = new Rectangle(level*LEVEL_OFFSET, 0, width - (level*LEVEL_OFFSET), treeControl.ColHeight);
		}

		public int Level {
			get { return level; }
			set {
				level = value;
				ComputeBounds();
			}
		}

		public TreeNode TreeNode {
			get { return treeNode; }
		}

		public bool Open {
			get { return open; }
			set {
				open = value;
				if (open) {
					ctlImage = minusImage;
				} else {
					ctlImage = plusImage;
				}
				InvalidatePaint();
			}
		}

		public bool Header {
			get { return header; }
			set {
				header = true;
				InvalidatePaint();
			}
		}

		protected override void Paint(UMD.HCIL.Piccolo.Util.PPaintContext paintContext) {
			base.Paint (paintContext);

			Graphics g = paintContext.Graphics;
			int x = level * LEVEL_OFFSET;
			if (ctlImage != null) {
				if (!header) {
					g.DrawImage(ctlImage, x, 0);
				}
			}
			x += CTLIMAGE_OFFSET;
			if (!header) {
				g.DrawImage(icon, x, 0);
			}
			x += ICON_OFFSET;
			for (int colNum=0; colNum<cols.Length; colNum++) {
				string col = cols[colNum];
				int colWidth = treeControl.ColWidths[colNum];
				if (colNum == 0) {
					colWidth -= (level * LEVEL_OFFSET);   // First column loses space for indentation
				}
				if (header) {
					ControlPaint.DrawButton(g, x, 0, colWidth, treeControl.ColHeight, ButtonState.Normal);
				}
				Rectangle layoutRect = new Rectangle(x, 0, colWidth, treeControl.ColHeight);
				StringFormat format = new StringFormat();
				g.DrawString(col, treeControl.Font, Brushes.Black, layoutRect, format);
				x += colWidth;
			}
		}
	}
}