///
/// A File System TreeModel.  Implements the TreeModel interface, and defines
/// a tree backed by the file system.  This file consists of three classes:
///   * FileSystemTreeModel  - The main tree model
///   * FileSystemTreeNode   - How individual files/directories are represented
///   * FileSystemHeaderNode - How the column header is represented
/// 
/// Ben Bederson
/// Human-Computer Interaction Lab
/// University of Maryland
/// May, 2005
/// 
using System;
using System.IO;
using System.Globalization;
using TreeControl;

namespace TreeControlTester {
	public class FileSystemTreeModel : TreeControl.TreeModel {
		private string root;

		public FileSystemTreeModel(string root) {
			this.root = root;
		}

		public TreeNode GetChild(TreeNode parent, int index) {
			TreeNode child = null;

			if (parent is FileSystemTreeNode) {
				FileSystemTreeNode fsNode = (FileSystemTreeNode)parent;
				if (fsNode.IsDir) {
					string path = fsNode.Path;
					DirectoryInfo dir = new DirectoryInfo(path);
					FileSystemInfo[] infos = dir.GetFileSystemInfos();
					child = new FileSystemTreeNode(infos[index].FullName);				
				}
			}

			return child;
		}

		public int GetChildCount(TreeNode parent) {
			int numChildren = 0;

			if (parent is FileSystemTreeNode) {
				FileSystemTreeNode fsNode = (FileSystemTreeNode)parent;
				if (fsNode.IsDir) {
					string path = fsNode.Path;
					DirectoryInfo dir = new DirectoryInfo(path);
					FileSystemInfo[] infos = dir.GetFileSystemInfos();
					numChildren = infos.Length;
				}
			}

			return numChildren;
		}

		public int GetIndexOfChild(TreeNode parent, TreeNode child) {
			int index = -1;

			FileSystemTreeNode fsParent = (FileSystemTreeNode)parent;
			FileSystemTreeNode fsChild = (FileSystemTreeNode)child;
			if (fsParent.IsDir) {
				string path = fsParent.Path;
				DirectoryInfo dir = new DirectoryInfo(path);
				FileSystemInfo[] infos = dir.GetFileSystemInfos();
				int childNum = 0;
				foreach (FileSystemInfo info in infos) {
					if (info.FullName.Equals(fsChild.Path)) {
						index = childNum;
						break;
					}
					childNum++;
				}
			}

			return index;
		}

		public TreeNode GetRoot() {
			return new FileSystemTreeNode(root);
		}

		public bool IsLeaf(TreeNode node) {
			bool isLeaf = true;

			if (node is FileSystemTreeNode) {
				FileSystemTreeNode fsNode = (FileSystemTreeNode)node;
				if (fsNode.IsDir) {
					isLeaf = false;
				}
			}

			return isLeaf;
		}

		public TreeNode GetColumnHeader() {
			return new FileSystemHeaderNode();
		}
	}

	public class FileSystemTreeNode : TreeControl.TreeNode {
		private string path;
		private bool isDir;

		public FileSystemTreeNode(string path) {
			this.path = path;
			if (Directory.Exists(path)) {
				isDir = true;
			} else {
				isDir = false;
			}
		}

		public int GetColumnCount() {
			return 3;
		}

		public string GetColumn(int index) {
			string col = "";

			switch (index) {
				case 0:
					if (isDir) {
						DirectoryInfo dir = new DirectoryInfo(path);
						col = dir.Name;
					} else {
						FileInfo file = new FileInfo(path);
						col = file.Name;
					}
					break;
				case 1:
					if (isDir) {
						col = "-";
					} else {
						FileInfo file = new FileInfo(path);
						NumberFormatInfo nfi = new CultureInfo( "en-US", false ).NumberFormat;
						nfi.CurrencySymbol = "";
						nfi.CurrencyDecimalDigits = 0;
						col = file.Length.ToString("C", nfi);
					}
					break;
				case 2:
					if (isDir) {
						DirectoryInfo dir = new DirectoryInfo(path);
						col = dir.LastWriteTime.ToShortDateString();
					} else {
						FileInfo file = new FileInfo(path);
						col = file.LastWriteTime.ToShortDateString();
					}
					break;
			}

			return col;
		}

		public bool IsDir {
			get { return isDir; }
		}

		public string Path {
			get { return path; }
		}
	}

	public class FileSystemHeaderNode : TreeNode {
		public FileSystemHeaderNode() {
		}

		public int GetColumnCount() {
			return 3;
		}

		public string GetColumn(int index) {
			string col = "";

			switch (index) {
				case 0:
					col = "Name                      ";
					break;
				case 1:
					col = "Size             ";
					break;
				case 2:
					col = "Date             ";
					break;
			}

			return col;
		}
	}
}